#ifndef _MULSERV_H
#define _MULSERV_H

/*-----------------------------------------------------------------------------
 Section: Macro Definitions
 ----------------------------------------------------------------------------*/
// 最大服务器数量
#define SERIVCE_MAX_COUNT   40

#ifdef	__cplusplus
extern "C"
{     
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/*-----------------------------------------------------------------------------
 Section: Type Definitions
 ----------------------------------------------------------------------------*/

/**
 * 请求与响应结果
**/
typedef enum{
    OK,                         // 设置成功
    ERROR,                      // 设置出错
    ERROR_IN_AUTO,              // 设置出错, 自动模式无法设置
    ERROR_ID_OVERFLOW,          // 设置出错, 请求设置的ID,越界
    ERROR_ID_DISCONNECT,        // 设置出错, 请求设置的ID主机, 已断开连接
    ERROR_CHECK_PROCESS,        // 设置出错, 检查进程失败，有进程未运行
    ERROR_STATUS                // 设置出错, 当前状态不能作为主机
} result_e;

/**
 * 交换机状态
*/
typedef enum{
    MUL_DISCONNECT,             // 交换机断开连接
    MUL_CONNECT,                // 交换机连接
} mul_switch_status_e;

/**
 * 主备状态
*/
typedef enum {
	MUL_OFFLINE   = 0x0,        // 脱机状态
	MUL_NOUSE     = 0x1,        // 未使用
	MUL_SLAVE     = 0x2,        // 备机
	MUL_MASTER    = 0x3,        // 主机
}  mul_status_e;

/**
 *  工作模式
*/
typedef enum {
	MUL_MANUAL =0x0,    // 手动
	MUL_AUTO = 0x1,     // 自动
    MUL_SW = 0x2,       // 表示手工与自动相互切换
} mul_mode_e;

#pragma pack(push, 1)
/**
 * 服务器状态信息
**/
typedef struct {
    int id;                                 // 服务器编号
    int master_id;                          // 主机服务器编号
    unsigned char mul_status : 4;           // 本机的主备状态 mul_status_e
    unsigned char work_mode  : 4;           // 工作模式      mul_mode_e
    unsigned char check_process_status;     // 检查进程状态, 0: 表示正常， 1表示有程序未运行
    int switch_count;                       // 交换机个数
    int swtich_statuss;                     // 交换机状态,每一位标识一台交换机,只有32位 mul_switch_status_e
} service_info_t;

/**
 * 响应 工作模式消息报文格式 或响应状态信息
*/
typedef struct 
{
    result_e result;     // 工作模式切换响应结果
} resp_sw_msg_t, 
  resp_set_msg_t,
  resp_msg_t;

/**
 * 响应 读取消息报文格式
*/
typedef struct 
{
    int count;                  // 主机数量
    int self_id;                // 当前服务器ID
    service_info_t serinfos[SERIVCE_MAX_COUNT]; // 服务器状态信息表
} resp_get_msg_t,
  msg_sts_t;

#pragma pack(pop)


/*-----------------------------------------------------------------------------
 Section: 一主多备 Function Definitions
 ----------------------------------------------------------------------------*/
 /**
 ******************************************************************************
 * @brief  等待双工（一主多备）状态改变
 * @retval 0: 成功, 其他表示失败
 ******************************************************************************
 */
 int waitMul();

 /**
 ******************************************************************************
 * @brief  获取双工（一主多备）状态
 * @param[out] resp_msg : 服务器状态
 * @retval result_e： 0表示成功, 其他表示失败
 ******************************************************************************
 */
int getMulSts(resp_get_msg_t * resp_msg);

/**
 ******************************************************************************
 * @brief   通过ID, 获取该id服务器的状态
 * @param[in]  id : 服务器id
 * @param[out] outinfo : 服务器状态
 * @retval  result_e： 0表示读取成功， 非0表示读取失败
 ******************************************************************************
 */
int getMulStsById(int id, service_info_t * outinfo);

 /**
 ******************************************************************************
 * @brief       请求切换主机id
 * @param[in]  id : 服务器id,要切换成的主机id
 * @retval result_e： 0表示成功, 其他表示失败
 ******************************************************************************
 */
int reqSwhMaster(int id);

/**
 ******************************************************************************
 * @brief      请求工作模式切换
 * @param[in]  e : 工作模式
 * @retval     0: 切换成功， 非0：切换失败
 ******************************************************************************
 */
int reqMulSwh(mul_mode_e e);

/**
 ******************************************************************************
 * @brief  请求切换工作模式
 * @retval 0: 成功, 其他表示失败
 ******************************************************************************
 */
int reqSwhMode();

/**
 ******************************************************************************
 * @brief   获取本机服务器主备状态
 * @param[out] mul_status_e : 本机服务器主备离线状态
 * @retval     0: 读取成功， 非0：读取失败
 ******************************************************************************
 */
int getSelfSts(mul_status_e * status);

/**
 ******************************************************************************
 * @brief      本机是否为主机	
 * @retval     0: 本机非主机， 1：本机为主机
 ******************************************************************************
 */
int isMaster();

/**
 ******************************************************************************
 * @brief      获取工作模式，自动模式，手动模式	
 * @param[out] mul_mode_e : 工作模式
 * @retval     0: 读取成功， 非0：读取失败
 ******************************************************************************
 */
int getMulMode(mul_mode_e * mode);


/*-----------------------------------------------------------------------------
 Section: 原双工 Function Definitions
 ----------------------------------------------------------------------------*/
/**
 ******************************************************************************
 * @brief  等待双工（一主多备）状态改变
 * @retval 0: 成功, 其他表示失败
 ******************************************************************************
 */
int waitDup(void);

 /**
 ******************************************************************************
 * @brief  获取双工（一主多备）状态
 * @param[out] msg_sts_t *dpsts : 服务器状态数组
 * @retval result_e： 0表示成功, 其他表示失败
 ******************************************************************************
 */
int getDupSts(msg_sts_t *dpsts);

/**
 ******************************************************************************
 * @brief   请求双工（一主多备）状态切换
 * @retval  result_e: 0表示成功, 其他表示失败
 ******************************************************************************
 */
int reqDupSwh(int id);

/**
 ******************************************************************************
 * @brief: 显示process状态
 * @retval  result_e: 无
 ******************************************************************************
 */
void showSelfProcess();


int is_integer(char *str);
#ifdef	__cplusplus
}
#endif

#endif
